=== modified file 'HACKING'
--- HACKING	2006-10-11 01:17:15 +0000
+++ HACKING	2006-10-13 01:31:59 +0000
@@ -209,6 +209,15 @@
 > factory, then yes, foo_factory is what I would use.
 
 
+Registries
+----------
+
+Several places in Bazaar use (or will use) a registry, which is a 
+mapping from names to objects or classes.  The registry allows for 
+loading in registered code only when it's needed, and keeping
+associated information such as a help string or description.
+
+
 Lazy Imports
 ------------
 

=== modified file 'NEWS'
--- NEWS	2006-10-12 00:51:31 +0000
+++ NEWS	2006-10-13 01:31:59 +0000
@@ -1,6 +1,7 @@
 IN DEVELOPMENT
 
   IMPROVEMENTS:
+
     * ``bzr help commands`` output is now shorter (Aaron Bentley)
 
     * ``bzr`` now uses lazy importing to reduce the startup time. This has
@@ -8,6 +9,12 @@
       little to do. For example ``bzr rocks`` time is down to 116ms from
       283ms. (John Arbash Meinel)
 
+    * New Registry class to provide name-to-object registry-like support,
+      for example for schemes where plugins can register new classes to
+      do certain tasks (e.g. log formatters). Also provides lazy registration
+      to allow modules to be loaded on request. (John Arbash Meinel, Adeodato
+      Simó)
+
   INTERNALS:
 
     * ``TestCase.run_bzr``, ``run_bzr_captured``, and ``run_bzr_subprocess``

=== added file 'bzrlib/registry.py'
--- bzrlib/registry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/registry.py	2006-10-13 00:55:26 +0000
@@ -0,0 +1,214 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Classes to provide name-to-object registry-like support."""
+
+
+class _ObjectGetter(object):
+    """Maintain a reference to an object, and return the object on request.
+
+    This is used by Registry to make plain objects function similarly
+    to lazily imported objects.
+
+    Objects can be any sort of python object (class, function, module,
+    instance, etc)
+    """
+
+    __slots__ = ['_obj']
+
+    def __init__(self, obj):
+        self._obj = obj
+
+    def get_obj(self):
+        """Get the object that was saved at creation time"""
+        return self._obj
+
+
+class _LazyObjectGetter(_ObjectGetter):
+    """Keep a record of a possible object.
+
+    When requested, load and return it.
+    """
+
+    __slots__ = ['_module_name', '_member_name', '_imported']
+
+    def __init__(self, module_name, member_name):
+        self._module_name = module_name
+        self._member_name = member_name
+        self._imported = False
+        super(_LazyObjectGetter, self).__init__(None)
+
+    def get_obj(self):
+        """Get the referenced object.
+
+        Upon first request, the object will be imported. Future requests will
+        return the imported object.
+        """
+        if not self._imported:
+            self._do_import()
+        return super(_LazyObjectGetter, self).get_obj()
+
+    def _do_import(self):
+        obj = __import__(self._module_name, globals(), locals(),
+                         [self._member_name])
+        if self._member_name:
+            obj = getattr(obj, self._member_name)
+        self._obj = obj
+        self._imported = True
+
+
+class Registry(object):
+    """A class that registers objects to a name.
+
+    There are many places that want to collect related objects and access them
+    by a key. This class is designed to allow registering the mapping from key
+    to object. It goes one step further, and allows registering a name to a
+    hypothetical object which has not been imported yet. It also supports
+    adding additional information at registration time so that decisions can be
+    made without having to import the object (which may be expensive).
+
+    The functions 'get', 'get_info', and 'get_help' also support a
+    'default_key' (settable through my_registry.default_key = XXX, XXX must
+    already be registered.) Calling my_registry.get() or my_registry.get(None),
+    will return the entry for the default key.
+    """
+
+    def __init__(self):
+        """Create a new Registry."""
+        self._default_key = None
+        # Map from key => (is_lazy, info)
+        self._dict = {}
+        self._help_dict = {}
+        self._info_dict = {}
+
+    def register(self, key, obj, help=None, info=None,
+                 override_existing=False):
+        """Register a new object to a name.
+
+        :param key: This is the key to use to request the object later.
+        :param obj: The object to register.
+        :param help: Help text for this entry. This may be a string or
+                a callable. If it is a callable, it should take two
+                parameters (registry, key): this registry and the key that 
+                the help was registered under.
+        :param info: More information for this entry. Registry.get_info()
+                can be used to get this information. Registry treats this as an
+                opaque storage location (it is defined by the caller).
+        :param override_existing: Raise KeyErorr if False and something has
+                already been registered for that key. If True, ignore if there
+                is an existing key (always register the new value).
+        """
+        if not override_existing:
+            if key in self._dict:
+                raise KeyError('Key %r already registered' % key)
+        self._dict[key] = _ObjectGetter(obj)
+        self._add_help_and_info(key, help=help, info=info)
+
+    def register_lazy(self, key, module_name, member_name,
+                      help=None, info=None,
+                      override_existing=False):
+        """Register a new object to be loaded on request.
+
+        :param module_name: The python path to the module. Such as 'os.path'.
+        :param member_name: The member of the module to return.  If empty or 
+                None, get() will return the module itself.
+        :param help: Help text for this entry. This may be a string or
+                a callable.
+        :param info: More information for this entry. Registry 
+        :param override_existing: If True, replace the existing object
+                with the new one. If False, if there is already something
+                registered with the same key, raise a KeyError
+        """
+        if not override_existing:
+            if key in self._dict:
+                raise KeyError('Key %r already registered' % key)
+        self._dict[key] = _LazyObjectGetter(module_name, member_name)
+        self._add_help_and_info(key, help=help, info=info)
+
+    def _add_help_and_info(self, key, help=None, info=None):
+        """Add the help and information about this key"""
+        self._help_dict[key] = help
+        self._info_dict[key] = info
+
+    def get(self, key=None):
+        """Return the object register()'ed to the given key.
+
+        May raise ImportError if the object was registered lazily and
+        there are any problems, or AttributeError if the module does not 
+        have the supplied member.
+
+        :param key: The key to obtain the object for. If no object has been
+            registered to that key, the object registered for self.default_key
+            will be returned instead, if it exists. Otherwise KeyError will be
+            raised.
+        :return: The previously registered object.
+        :raises ImportError: If the object was registered lazily, and there are
+            problems during import.
+        :raises AttributeError: If registered lazily, and the module does not
+            contain the registered member.
+        """
+        return self._dict[self._get_key_or_default(key)].get_obj()
+
+    def _get_key_or_default(self, key=None):
+        """Return either 'key' or the default key if key is None"""
+        if key is not None:
+            return key
+        if self.default_key is None:
+            raise KeyError('Key is None, and no default key is set')
+        else:
+            return self.default_key
+
+    def get_help(self, key=None):
+        """Get the help text associated with the given key"""
+        the_help = self._help_dict[self._get_key_or_default(key)]
+        if callable(the_help):
+            return the_help(self, key)
+        return the_help
+
+    def get_info(self, key=None):
+        """Get the extra information associated with the given key"""
+        return self._info_dict[self._get_key_or_default(key)]
+
+    def remove(self, key):
+        """Remove a registered entry.
+
+        This is mostly for the test suite, but it can be used by others
+        """
+        del self._dict[key]
+
+    def __contains__(self, key):
+        return key in self._dict
+
+    def keys(self):
+        """Get a list of registered entries"""
+        return sorted(self._dict.keys())
+
+    def iteritems(self):
+        for key, getter in self._dict.iteritems():
+            yield key, getter.get_obj()
+
+    def _set_default_key(self, key):
+        if not self._dict.has_key(key):
+            raise KeyError('No object registered under key %s.' % key)
+        else:
+            self._default_key = key
+
+    def _get_default_key(self):
+        return self._default_key
+
+    default_key = property(_get_default_key, _set_default_key,
+                            doc="Current value of the default key."
+                                " Can be set to any existing key.")

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-10-11 00:23:23 +0000
+++ bzrlib/tests/__init__.py	2006-10-13 01:31:59 +0000
@@ -1633,6 +1633,7 @@
                    'bzrlib.tests.test_plugins',
                    'bzrlib.tests.test_progress',
                    'bzrlib.tests.test_reconcile',
+                   'bzrlib.tests.test_registry',
                    'bzrlib.tests.test_repository',
                    'bzrlib.tests.test_revert',
                    'bzrlib.tests.test_revision',

=== added file 'bzrlib/tests/test_registry.py'
--- bzrlib/tests/test_registry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_registry.py	2006-09-21 22:25:19 +0000
@@ -0,0 +1,262 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the Registry classes"""
+
+import os
+import sys
+
+from bzrlib import (
+    errors,
+    registry,
+    osutils,
+    )
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+
+class TestRegistry(TestCase):
+
+    def register_stuff(self, a_registry):
+        a_registry.register('one', 1)
+        a_registry.register('two', 2)
+        a_registry.register('four', 4)
+        a_registry.register('five', 5)
+
+    def test_registry(self):
+        a_registry = registry.Registry()
+        self.register_stuff(a_registry)
+
+        self.failUnless(a_registry.default_key is None)
+
+        # test get() (self.default_key == None)
+        self.assertRaises(KeyError, a_registry.get)
+        self.assertRaises(KeyError, a_registry.get, None)
+        self.assertEqual(2, a_registry.get('two'))
+        self.assertRaises(KeyError, a_registry.get, 'three')
+
+        # test _set_default_key
+        a_registry.default_key = 'five'
+        self.failUnless(a_registry.default_key == 'five')
+        self.assertEqual(5, a_registry.get())
+        self.assertEqual(5, a_registry.get(None))
+        # If they ask for a specific entry, they should get KeyError
+        # not the default value. They can always pass None if they prefer
+        self.assertRaises(KeyError, a_registry.get, 'six')
+        self.assertRaises(KeyError, a_registry._set_default_key, 'six')
+
+        # test keys()
+        self.assertEqual(['five', 'four', 'one', 'two'], a_registry.keys())
+
+    def test_registry_funcs(self):
+        a_registry = registry.Registry()
+        self.register_stuff(a_registry)
+
+        self.failUnless('one' in a_registry)
+        a_registry.remove('one')
+        self.failIf('one' in a_registry)
+        self.assertRaises(KeyError, a_registry.get, 'one')
+
+        a_registry.register('one', 'one')
+
+        self.assertEqual(['five', 'four', 'one', 'two'],
+                         sorted(a_registry.keys()))
+        self.assertEqual([('five', 5), ('four', 4),
+                          ('one', 'one'), ('two', 2)],
+                         sorted(a_registry.iteritems()))
+
+    def test_register_override(self):
+        a_registry = registry.Registry()
+        a_registry.register('one', 'one')
+        self.assertRaises(KeyError, a_registry.register, 'one', 'two')
+        self.assertRaises(KeyError, a_registry.register, 'one', 'two',
+                                    override_existing=False)
+
+        a_registry.register('one', 'two', override_existing=True)
+        self.assertEqual('two', a_registry.get('one'))
+
+        self.assertRaises(KeyError, a_registry.register_lazy,
+                          'one', 'three', 'four')
+
+        a_registry.register_lazy('one', 'module', 'member',
+                                 override_existing=True)
+
+    def test_registry_help(self):
+        a_registry = registry.Registry()
+        a_registry.register('one', 1, help='help text for one')
+        # We should not have to import the module to return the help
+        # information
+        a_registry.register_lazy('two', 'nonexistent_module', 'member',
+                                 help='help text for two')
+
+        # We should be able to handle a callable to get information
+        help_calls = []
+        def generic_help(reg, key):
+            help_calls.append(key)
+            return 'generic help for %s' % (key,)
+        a_registry.register('three', 3, help=generic_help)
+        a_registry.register_lazy('four', 'nonexistent_module', 'member2',
+                                 help=generic_help)
+        a_registry.register('five', 5)
+
+        def help_from_object(reg, key):
+            obj = reg.get(key)
+            return obj.help()
+
+        class SimpleObj(object):
+            def help(self):
+                return 'this is my help'
+        a_registry.register('six', SimpleObj(), help=help_from_object)
+
+        self.assertEqual('help text for one', a_registry.get_help('one'))
+        self.assertEqual('help text for two', a_registry.get_help('two'))
+        self.assertEqual('generic help for three',
+                         a_registry.get_help('three'))
+        self.assertEqual(['three'], help_calls)
+        self.assertEqual('generic help for four',
+                         a_registry.get_help('four'))
+        self.assertEqual(['three', 'four'], help_calls)
+        self.assertEqual(None, a_registry.get_help('five'))
+        self.assertEqual('this is my help', a_registry.get_help('six'))
+
+        self.assertRaises(KeyError, a_registry.get_help, None)
+        self.assertRaises(KeyError, a_registry.get_help, 'seven')
+
+        a_registry.default_key = 'one'
+        self.assertEqual('help text for one', a_registry.get_help(None))
+        self.assertRaises(KeyError, a_registry.get_help, 'seven')
+
+        self.assertEqual([('five', None),
+                          ('four', 'generic help for four'),
+                          ('one', 'help text for one'),
+                          ('six', 'this is my help'),
+                          ('three', 'generic help for three'),
+                          ('two', 'help text for two'),
+                         ], sorted((key, a_registry.get_help(key))
+                                    for key in a_registry.keys()))
+
+        # We don't know what order it was called in, but we should get
+        # 2 more calls to three and four
+        self.assertEqual(['four', 'four', 'three', 'three'],
+                         sorted(help_calls))
+
+    def test_registry_info(self):
+        a_registry = registry.Registry()
+        a_registry.register('one', 1, info='string info')
+        # We should not have to import the module to return the info
+        a_registry.register_lazy('two', 'nonexistent_module', 'member',
+                                 info=2)
+
+        # We should be able to handle a callable to get information
+        a_registry.register('three', 3, info=['a', 'list'])
+        obj = object()
+        a_registry.register_lazy('four', 'nonexistent_module', 'member2',
+                                 info=obj)
+        a_registry.register('five', 5)
+
+        self.assertEqual('string info', a_registry.get_info('one'))
+        self.assertEqual(2, a_registry.get_info('two'))
+        self.assertEqual(['a', 'list'], a_registry.get_info('three'))
+        self.assertIs(obj, a_registry.get_info('four'))
+        self.assertIs(None, a_registry.get_info('five'))
+
+        self.assertRaises(KeyError, a_registry.get_info, None)
+        self.assertRaises(KeyError, a_registry.get_info, 'six')
+
+        a_registry.default_key = 'one'
+        self.assertEqual('string info', a_registry.get_info(None))
+        self.assertRaises(KeyError, a_registry.get_info, 'six')
+
+        self.assertEqual([('five', None),
+                          ('four', obj),
+                          ('one', 'string info'),
+                          ('three', ['a', 'list']),
+                          ('two', 2),
+                         ], sorted((key, a_registry.get_info(key))
+                                    for key in a_registry.keys()))
+
+
+class TestRegistryWithDirs(TestCaseInTempDir):
+    """Registry tests that require temporary dirs"""
+
+    def create_plugin_file(self, contents):
+        """Create a file to be used as a plugin.
+
+        This is created in a temporary directory, so that we
+        are sure that it doesn't start in the plugin path.
+        """
+        os.mkdir('tmp')
+        plugin_name = 'bzr_plugin_a_%s' % (osutils.rand_chars(4),)
+        open('tmp/'+plugin_name+'.py', 'wb').write(contents)
+        return plugin_name
+
+    def create_simple_plugin(self):
+        return self.create_plugin_file(
+            'object1 = "foo"\n'
+            '\n\n'
+            'def function(a,b,c):\n'
+            '    return a,b,c\n'
+            '\n\n'
+            'class MyClass(object):\n'
+            '    def __init__(self, a):\n'
+            '      self.a = a\n'
+            '\n\n'
+        )
+
+    def test_lazy_import_registry(self):
+        plugin_name = self.create_simple_plugin()
+        a_registry = registry.Registry()
+        a_registry.register_lazy('obj', plugin_name, 'object1')
+        a_registry.register_lazy('function', plugin_name, 'function')
+        a_registry.register_lazy('klass', plugin_name, 'MyClass')
+        a_registry.register_lazy('module', plugin_name, None)
+
+        self.assertEqual(['function', 'klass', 'module', 'obj'],
+                         sorted(a_registry.keys()))
+        # The plugin should not be loaded until we grab the first object
+        self.failIf(plugin_name in sys.modules)
+
+        # By default the plugin won't be in the search path
+        self.assertRaises(ImportError, a_registry.get, 'obj')
+
+        plugin_path = os.getcwd() + '/tmp'
+        sys.path.append(plugin_path)
+        try:
+            obj = a_registry.get('obj')
+            self.assertEqual('foo', obj)
+            self.failUnless(plugin_name in sys.modules)
+
+            # Now grab another object
+            func = a_registry.get('function')
+            self.assertEqual(plugin_name, func.__module__)
+            self.assertEqual('function', func.__name__)
+            self.assertEqual((1, [], '3'), func(1, [], '3'))
+
+            # And finally a class
+            klass = a_registry.get('klass')
+            self.assertEqual(plugin_name, klass.__module__)
+            self.assertEqual('MyClass', klass.__name__)
+
+            inst = klass(1)
+            self.assertIsInstance(inst, klass)
+            self.assertEqual(1, inst.a)
+
+            module = a_registry.get('module')
+            self.assertIs(obj, module.object1)
+            self.assertIs(func, module.function)
+            self.assertIs(klass, module.MyClass)
+        finally:
+            sys.path.remove(plugin_path)
+

